@@ -771,11 +771,7 @@ impl GeP3 {
         let y_squared = y.square();
         let u = y_squared - FE_ONE;
         let v = (y_squared * FE_D) + FE_ONE;
-        let v_raise_3 = v.square() * v;
-        let v_raise_7 = v_raise_3.square() * v;
-        let uv7 = v_raise_7 * u; // Is this commutative? u comes second in the code, but not in the notation...
-
-        let mut x = uv7.pow25523() * v_raise_3 * u;
+        let mut x = (u * v).pow25523() * u;
 
         let vxx = x.square() * v;
         let check = vxx - u;
@@ -941,23 +937,6 @@ pub fn sc_reduce(s: &mut [u8]) {
     let s21: i64 = 2097151 & (load_3i(&s[55..58]) >> 1);
     let s22: i64 = 2097151 & (load_4i(&s[57..61]) >> 6);
     let s23: i64 = load_4i(&s[60..64]) >> 3;
-    let mut carry0: i64;
-    let mut carry1: i64;
-    let mut carry2: i64;
-    let mut carry3: i64;
-    let mut carry4: i64;
-    let mut carry5: i64;
-    let mut carry6: i64;
-    let mut carry7: i64;
-    let mut carry8: i64;
-    let mut carry9: i64;
-    let mut carry10: i64;
-    let mut carry11: i64;
-    let carry12: i64;
-    let carry13: i64;
-    let carry14: i64;
-    let carry15: i64;
-    let carry16: i64;
 
     s11 += s23 * 666643;
     s12 += s23 * 470296;
@@ -1001,38 +980,38 @@ pub fn sc_reduce(s: &mut [u8]) {
     s10 += s18 * 136657;
     s11 -= s18 * 683901;
 
-    carry6 = (s6 + (1 << 20)) >> 21;
+    let mut carry6: i64 = (s6 + (1 << 20)) >> 21;
     s7 += carry6;
     s6 -= carry6 << 21;
-    carry8 = (s8 + (1 << 20)) >> 21;
+    let mut carry8: i64 = (s8 + (1 << 20)) >> 21;
     s9 += carry8;
     s8 -= carry8 << 21;
-    carry10 = (s10 + (1 << 20)) >> 21;
+    let mut carry10: i64 = (s10 + (1 << 20)) >> 21;
     s11 += carry10;
     s10 -= carry10 << 21;
-    carry12 = (s12 + (1 << 20)) >> 21;
+    let carry12: i64 = (s12 + (1 << 20)) >> 21;
     s13 += carry12;
     s12 -= carry12 << 21;
-    carry14 = (s14 + (1 << 20)) >> 21;
+    let carry14: i64 = (s14 + (1 << 20)) >> 21;
     s15 += carry14;
     s14 -= carry14 << 21;
-    carry16 = (s16 + (1 << 20)) >> 21;
+    let carry16: i64 = (s16 + (1 << 20)) >> 21;
     s17 += carry16;
     s16 -= carry16 << 21;
 
-    carry7 = (s7 + (1 << 20)) >> 21;
+    let mut carry7: i64 = (s7 + (1 << 20)) >> 21;
     s8 += carry7;
     s7 -= carry7 << 21;
-    carry9 = (s9 + (1 << 20)) >> 21;
+    let mut carry9: i64 = (s9 + (1 << 20)) >> 21;
     s10 += carry9;
     s9 -= carry9 << 21;
-    carry11 = (s11 + (1 << 20)) >> 21;
+    let mut carry11: i64 = (s11 + (1 << 20)) >> 21;
     s12 += carry11;
     s11 -= carry11 << 21;
-    carry13 = (s13 + (1 << 20)) >> 21;
+    let carry13: i64 = (s13 + (1 << 20)) >> 21;
     s14 += carry13;
     s13 -= carry13 << 21;
-    carry15 = (s15 + (1 << 20)) >> 21;
+    let carry15: i64 = (s15 + (1 << 20)) >> 21;
     s16 += carry15;
     s15 -= carry15 << 21;
 
@@ -1079,13 +1058,13 @@ pub fn sc_reduce(s: &mut [u8]) {
     s5 -= s12 * 683901;
     s12 = 0;
 
-    carry0 = (s0 + (1 << 20)) >> 21;
+    let mut carry0: i64 = (s0 + (1 << 20)) >> 21;
     s1 += carry0;
     s0 -= carry0 << 21;
-    carry2 = (s2 + (1 << 20)) >> 21;
+    let mut carry2: i64 = (s2 + (1 << 20)) >> 21;
     s3 += carry2;
     s2 -= carry2 << 21;
-    carry4 = (s4 + (1 << 20)) >> 21;
+    let mut carry4: i64 = (s4 + (1 << 20)) >> 21;
     s5 += carry4;
     s4 -= carry4 << 21;
     carry6 = (s6 + (1 << 20)) >> 21;
@@ -1098,13 +1077,13 @@ pub fn sc_reduce(s: &mut [u8]) {
     s11 += carry10;
     s10 -= carry10 << 21;
 
-    carry1 = (s1 + (1 << 20)) >> 21;
+    let mut carry1: i64 = (s1 + (1 << 20)) >> 21;
     s2 += carry1;
     s1 -= carry1 << 21;
-    carry3 = (s3 + (1 << 20)) >> 21;
+    let mut carry3: i64 = (s3 + (1 << 20)) >> 21;
     s4 += carry3;
     s3 -= carry3 << 21;
-    carry5 = (s5 + (1 << 20)) >> 21;
+    let mut carry5: i64 = (s5 + (1 << 20)) >> 21;
     s6 += carry5;
     s5 -= carry5 << 21;
     carry7 = (s7 + (1 << 20)) >> 21;
